package io.tswf.minecraft.blockventure.core.gui

import groovy.transform.CompileStatic
import groovy.util.logging.Slf4j
import io.tswf.groovy.bettergroovy.transforms.validation.checkedmapconstructor.CheckedMapConstructor
import io.tswf.minecraft.blockventure.core.api.annotation.MapNBT
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import io.tswf.minecraft.blockventure.core.api.network.packet.NBTSerializablePlayNetworkPacket
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.nbt.NbtElement
import net.minecraft.world.World

@CheckedMapConstructor
@Slf4j
@CompileStatic
class OpenScreenAtClientPacket extends NBTSerializablePlayNetworkPacket {

    public static final String PACKET_ID = "blockventure:open_screen_at_gui_packet"

    @MapNBT
    int xCoord

    @MapNBT
    int yCoord

    @MapNBT
    int zCoord

    @MapNBT
    String customDataClass = null

    @MapNBT
    NbtElement customData = null

    @MapNBT
    int windowId

    @MapNBT
    String guiScreenId

    @MapNBT
    String modId

    @Override
    void handleOnClient(World localWorld, PlayerEntity localPlayer) {
        if (customData != null && customDataClass == null) {
            log.warn("There is no custom data class was set for custom data nbt: ${customData}! Skipping...")
        }

        def customDataObject = null
        if (customData != null && customDataClass != null) {
            customDataObject = NBTObjectMapper.mapToObject(customData, ClassLoader.contextClassloader().loadClass(customDataClass))
        }

        PlayerScreenService.openScreen(modId, localPlayer, guiScreenId, windowId, localWorld, xCoord, yCoord, zCoord, customDataObject)
    }

    @Override
    boolean isBackgroundOnClient() {
        return false
    }
}
