package io.tswf.minecraft.blockventure.core.world.storage

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import io.tswf.minecraft.blockventure.core.api.world.storage.IWorldCustomDataStorage
import net.minecraft.nbt.NbtCompound
import net.minecraft.server.world.ServerWorld

import java.lang.reflect.Type

@CompileStatic
class NBTServerWorldCustomDataStorage implements IWorldCustomDataStorage {
    NbtCompound compound
    String name
    ServerWorld world

    NBTServerWorldCustomDataStorage(NbtCompound compound, String name, ServerWorld world) {
        this.compound = compound
        this.name = name
        this.world = world
    }

    @Override
    def <T> T readValue(String name, Class<T> classOfT) {
        return readValue(name, (Type) classOfT)
    }

    @Override
    def <T> T readValue(String name, Type typeOfT) {
        if (!compound.contains(name)) {
            return null
        }

        def tag = compound.get(name)
        def mappedObject = NBTObjectMapper.mapToObject(tag, typeOfT)

        return (T) mappedObject
    }

    @Override
    void commit() {
        NBTServerWorldCustomDataStorageIO.commitStorage(this)
    }

    @Override
    void writeValue(String name, Object value) {
        writeValue(name, value, value?.class)
    }

    @Override
    void writeValue(String name, Object value, Type typeOfValue) {
        def serializedNbt = NBTObjectMapper.mapToNbt(value, typeOfValue)
        compound.put(name, serializedNbt)
    }
}
