package io.tswf.minecraft.blockventure.core.world.storage

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.transmutation.view.ViewMinecraftServer
import net.minecraft.nbt.NbtCompound
import net.minecraft.nbt.NbtIo
import net.minecraft.server.world.ServerWorld

@CompileStatic
class NBTServerWorldCustomDataStorageIO {

    public static final String BLOCKVENTURE_MODS_CUSTOM_DATA_FOLDER_PATH = "blockventure-mods-custom-data"
    public static final String NBT_CUSTOM_DATA_STORAGE_PATTERN = "nbt-storage-%s.dat"

    static NBTServerWorldCustomDataStorage newStorage(ServerWorld world, String name, boolean persistEmpty) {
        def customDataFile = getCustomDataFile(world, name)

        if (customDataFile.exists() && customDataFile.length() > 0) {
            def readedCustomDataCompound = NbtIo.readCompressed(customDataFile)
            return new NBTServerWorldCustomDataStorage(readedCustomDataCompound, name, world)
        } else {
            def newCreatedStorage = new NBTServerWorldCustomDataStorage(new NbtCompound(), name, world)
            if (persistEmpty) {
                commitStorage(newCreatedStorage)
            }

            return newCreatedStorage
        }
    }

    static void commitStorage(NBTServerWorldCustomDataStorage storage) {
        def customDataFile = getCustomDataFile(storage.world, storage.name)
        customDataFile.createParentDirectories()
        NbtIo.writeCompressed(storage.compound, customDataFile)
    }

    static File getCustomDataFile(ServerWorld world, String name) {
        def server = world.server
        def viewServer = (ViewMinecraftServer) server
        def session = viewServer.session

        def saveDirFile = session.getWorldDirectory(world.registryKey).toFile()
        if (saveDirFile == null) {
            throw new IllegalStateException("There is no save dir in server world: ${world}!")
        }

        def saveDirAbsFile = saveDirFile.absoluteCanonicalFile
        def customDataDir = saveDirAbsFile.subFile(BLOCKVENTURE_MODS_CUSTOM_DATA_FOLDER_PATH)
        def customDataFile = customDataDir.subFile(getNbtCustomDataFileName(name))

        return customDataFile
    }

    static String getNbtCustomDataFileName(String key) {
        return NBT_CUSTOM_DATA_STORAGE_PATTERN.formatString(key)
    }

}
